package jp.ac.uaizu.rdbs.common;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ControllerAdvice;

import jp.ac.uaizu.rdbs.common.RDBSConstants.LoginSession;
/**
 * Global class for login information using session
 * @author h.kikuchi
 *
 */
@ControllerAdvice
public class GlobalLoginSession {	
	/**
	 * Check the login status saved in the session.
	 * @param request Request object to get session.
	 * @return login status (true:Login false:Logout)
	 */
	public boolean checkSessionState(HttpServletRequest request) {
		boolean result = false;
		String loginState = LoginSession.LOGIN_STATE_DISABLE;
		
		HttpSession session = request.getSession(false);
		
		if(session == null) {
			session = request.getSession(true);
			session.setAttribute(LoginSession.LOGIN_STATE_KEY, LoginSession.LOGIN_STATE_DISABLE);
			return result;
		}
		else {
			loginState = (String)session.getAttribute(LoginSession.LOGIN_STATE_KEY);
		}
		
		if(LoginSession.LOGIN_STATE_ENABLE.equals(loginState)) {
			result = true;
		}
		
		return result;
	}
	
	/**
	 * Check the login status and perform the necessary settings for logout
	 * @param request Request object to get session
	 * @param model Object that sets the attributes required for Logout
	 * @return Logout setting result (true: set, false: not set)
	 */
	public boolean logoutAttrSetDecs(HttpServletRequest request, Model model) {
		boolean result = false;
		
		if (!checkSessionState(request)) {
			setLoginAttr(model, LoginSession.ATTRIBUTE_SET_LOGOUT);
			result = true;
		}
		
		return result;
	}
	
	/**
	 * Implement necessary login / logout screen settings
	 * @param model Object that sets the attributes required for Logout
	 * @param setAttr Login status judgment constant
	 */
	public void setLoginAttr(Model model, boolean setAttr) {
		
		if (setAttr == LoginSession.ATTRIBUTE_SET_LOGIN) {
			model.addAttribute("ButtonActive", true);
			model.addAttribute("LoginState", "Logout");
			model.addAttribute("LoginLink", "Top/Logout");
		}
		else {
			model.addAttribute("ButtonActive", false);
			model.addAttribute("LoginState", "Login");
			model.addAttribute("LoginLink", "Login");
		}
	}
}
