package jp.ac.uaizu.rdbs.controller;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.zip.ZipOutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.attribute.PosixFilePermissions;
import javax.servlet.http.HttpServletRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import jp.ac.uaizu.rdbs.common.GlobalLoginSession;
import jp.ac.uaizu.rdbs.common.RDBSConstants;
import jp.ac.uaizu.rdbs.dao.MapFileInfoDao;
import jp.ac.uaizu.rdbs.form.DownloadForm;
import jp.ac.uaizu.rdbs.service.DownloaderService;

import org.apache.commons.lang3.RandomStringUtils;

/**
 * Controller class corresponding to the search screen.
 */
@Controller
public class DownloaderController {
	private Logger logger = LogManager.getLogger(DownloaderController.class);
	
	@Autowired
	GlobalLoginSession globalLoginSession;
	
	@Autowired
	private DownloaderService downloaderService;
	
	@Autowired
	private Properties applicationProperties;
	
	@InitBinder
    public void dateBinder(WebDataBinder binder) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        CustomDateEditor editor = new CustomDateEditor(dateFormat, true);
        
        SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");
        CustomDateEditor timeEditor = new CustomDateEditor(timeFormat, true);
        
        binder.registerCustomEditor(Date.class, editor);
        binder.registerCustomEditor(Date.class, timeEditor);
    }
	
    @ModelAttribute("DownloadForm")
    public DownloadForm setUpDownloadForm() {
    	DownloadForm form = new DownloadForm();
    	return form;
    }
	
    /**
     * Displays the search screen.
     * @param request Request object to get session.
     * @param model Data used between View and Controller.
     * @return JSP file name.
     */
	@RequestMapping(value = "downloader", method = RequestMethod.GET)
	public String showDownloader(HttpServletRequest request, Model model) {
		if (globalLoginSession.logoutAttrSetDecs(request, model)) {
			return "redirect:Top";
		}
		String errMsg = (String)request.getAttribute("ErrorMsg");
		if ((errMsg == null) || (errMsg.isEmpty())) {
			model.addAttribute("ErrorMsg", "");
		}
		
		return "Downloader";
	}
	

    /**
     * Download the package corresponding to the passed package ID.
     * @param packageId Requested package ID list.
     * @param request Request object to get session. 
     * @param respose Response object. 
     */
    @ResponseBody
    @RequestMapping(value = "download", method = RequestMethod.POST, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, 
            produces = {MediaType.APPLICATION_ATOM_XML_VALUE, MediaType.APPLICATION_JSON_VALUE})
    public Map<String,Object> requestDownload(@RequestParam("robot_id") String robotId, @RequestParam("location") String location,
    		@RequestParam("date_from") String dateFrom, @RequestParam("date_to") String dateTo) {

    	Map<String,Object> downloadFile = new HashMap<String, Object>();
    	
    	try {
    		List<MapFileInfoDao> mapFileInfoList = downloaderService.findMapInfo(robotId, location, dateFrom, dateTo);
    		
    		if(mapFileInfoList.size() > 0) {
	    		SimpleDateFormat inSdf = new SimpleDateFormat("yyyyMMddHHmmss");
	    		String fileName = inSdf.format(new Date()) + "_Map.zip";
	    		String rdmStr = RandomStringUtils.randomAlphanumeric(10);
	    		String dlPath = applicationProperties.getProperty(RDBSConstants.ApplicationProperties.PROP_NAME_DOWNLOAD_PATH) + "/" + rdmStr;
	    		File dfile = new File(dlPath);
	    		dfile.mkdir();
    			Files.setPosixFilePermissions(Paths.get(dlPath),  PosixFilePermissions.fromString("rwxr-xr-x"));

	    		String zipFilePath = dlPath + "/" + fileName; 
	    		InputStream inStream = null;
	    		ZipOutputStream outStream = new ZipOutputStream(new FileOutputStream(zipFilePath));
	    		logger.info(zipFilePath);
		    	for(MapFileInfoDao mapFileInfoElm: mapFileInfoList) {
		    		String yamlFilePath = applicationProperties.getProperty(RDBSConstants.ApplicationProperties.PROP_NAME_RDR_ROOT_PATH) + mapFileInfoElm.getYamlFilePath();
		    		String pgmFilePath = applicationProperties.getProperty(RDBSConstants.ApplicationProperties.PROP_NAME_RDR_ROOT_PATH) + mapFileInfoElm.getPgmFilePath();
		    		logger.info(yamlFilePath);
		    		logger.info(pgmFilePath);
		    		File yamlFile = new File(yamlFilePath);
		    		File pgmFile = new File(pgmFilePath);
		    		String yamlName = inSdf.format(mapFileInfoElm.getDatetime()) + "/" + mapFileInfoElm.getRobotId() + "/" + mapFileInfoElm.getLocation() + "/" + yamlFile.getName();
		    		String pgmName = inSdf.format(mapFileInfoElm.getDatetime()) + "/" + mapFileInfoElm.getRobotId() + "/" + mapFileInfoElm.getLocation() + "/" + pgmFile.getName();
		    				
		    		downloaderService.createZip(outStream, yamlName, yamlFile, inStream);
		    		downloaderService.createZip(outStream, pgmName, pgmFile, inStream);
		    	}
		    	outStream.close();
	
		    	/*Change permissions and enable reading*/
		    	Files.setPosixFilePermissions(Paths.get(zipFilePath),  PosixFilePermissions.fromString("rwxr-xr-x"));
	    		
		    	String zipContextPath = zipFilePath.replace(
	    				applicationProperties.getProperty(RDBSConstants.ApplicationProperties.PROP_NAME_DOWNLOAD_PATH),
	    				RDBSConstants.FixedValue.CONTEXT_PATH_DOWNLOAD_DATA);

		    	downloadFile.put("filePath", zipContextPath);
		    	downloadFile.put("fileName", fileName);
		    	downloadFile.put("count", mapFileInfoList.size());
    		}
    		else {
    	    	downloadFile.put("filePath", "");
    	    	downloadFile.put("fileName", "");
    	    	downloadFile.put("count", 0);
    		}
    	}
    	catch(Exception e) {
    		logger.error(e.getMessage());
    		return null;
    	}
       	
    	return downloadFile;
    }    
}
