package jp.ac.uaizu.rdbs.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import jp.ac.uaizu.rdbs.common.GlobalFormValidation;
import jp.ac.uaizu.rdbs.common.GlobalLoginSession;
import jp.ac.uaizu.rdbs.common.RDBSConstants.LoginSession;
import jp.ac.uaizu.rdbs.common.RDBSConstants.Message;
import jp.ac.uaizu.rdbs.form.LoginForm;
import jp.ac.uaizu.rdbs.service.LoginService;

/**
 * Controller class corresponding to the Login screen.
 */
@Controller
public class LoginController {

	private Logger logger = LogManager.getLogger(LoginController.class);

	@Autowired
	private LoginService loginService;
	
	@Autowired
	private GlobalLoginSession globalLoginSession;
	
	@Autowired
	private GlobalFormValidation globalFormValidation;

    @ModelAttribute("LoginForm")
    public LoginForm setUpLoginForm() {
    	LoginForm form = new LoginForm();
        return form;
    }

    /**
     * Displays the login screen.
     * @param model Data used between View and Controller.
     * @return  JSP file name
     */
	@RequestMapping(value = {"Login", "Top/Login"}, method = RequestMethod.GET)
	public String showLogin(HttpServletRequest request, Model model) {
		logger.info("LoginController/ShowLogin Start");
		
		String jspName = "";
		
		if (!globalLoginSession.checkSessionState(request)) {
			model.addAttribute("ErrorMsg", "");
			jspName = "Login";
		}
		else {
			globalLoginSession.setLoginAttr(model, LoginSession.ATTRIBUTE_SET_LOGIN);
			jspName = "redirect:Top";
		}
		logger.info("LoginController/ShowLogin End");
		
		return jspName;
	}

	/**
	 * Authenticates the user requested from the input form
	 * @param form Form input / output data.
	 * @param result Error information of input check result.
	 * @param request Request object to get session.
	 * @param model Data used between View and Controller.
	 * @return  JSP file name.
	 */
	@RequestMapping(value = "Login", method = RequestMethod.POST)
	public String login(@Valid @ModelAttribute LoginForm form, BindingResult result, HttpServletRequest request, Model model) {
		logger.info("LoginController/Login Start");
		
		String errorMsg = globalFormValidation.getErrorMsg(result);
    	if (!errorMsg.isEmpty()) {
    		model.addAttribute("ErrorMsg", errorMsg);
    		return "Login";
		}

		String loginID = form.getLoginId();
		String loginPass = form.getPasswordId();
		
		boolean authRes = loginService.loginAuth(loginID, loginPass);

		if(authRes != true) {
			logger.info("Login authentication failed");
			model.addAttribute("ErrorMsg", Message.MSG_ERR_001);
			return "Login";
		}
		
		/*Set login status for session*/
		HttpSession session = request.getSession(true);
		session.setAttribute(LoginSession.LOGIN_STATE_KEY, LoginSession.LOGIN_STATE_ENABLE);
		
		globalLoginSession.setLoginAttr(model, LoginSession.ATTRIBUTE_SET_LOGIN);
		
		logger.info("LoginController/Login End");
		
		return "redirect:Top";
	}
}
