package jp.ac.uaizu.rdbs.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import jp.ac.uaizu.rdbs.common.GlobalLoginSession;
import jp.ac.uaizu.rdbs.common.RDBSConstants.LoginSession;
import jp.ac.uaizu.rdbs.form.LoginForm;

/**
 * Controller class corresponding to the top screen.
 */
@Controller
public class TopController {

	private Logger logger = LogManager.getLogger(TopController.class);
	
	@Autowired
	GlobalLoginSession globalLoginSession;
	
	@ModelAttribute("LoginForm")
	public LoginForm setUpLoginForm() {
		LoginForm form = new LoginForm();
		return form;
	}


	/**
	 * Displays the top screen.
	 * Switch between Login and Logout links according to login status.
	 * @param request Request object to get session.
	 * @param model Data used between View and Controller.
	 * @return JSP file name.
	 */
    @RequestMapping(value = "/Top", method = RequestMethod.GET)
    public String showTop(HttpServletRequest request, Model model) {
    	logger.info("TopController/ShowTop Start");
    	String result = "Top";
    	
		if (globalLoginSession.checkSessionState(request)) {
			globalLoginSession.setLoginAttr(model, LoginSession.ATTRIBUTE_SET_LOGIN);
		}
		else {
			globalLoginSession.setLoginAttr(model, LoginSession.ATTRIBUTE_SET_LOGOUT);
			result = "Login";
		}
    	logger.info("TopController/ShowTop End");
    	
    	return result;
    }
    
    /**
     * Change login state to unlogged state
	 * @param request Request object to get session.
	 * @param model Data used between View and Controller
     * @return JSP file name
     */
    @RequestMapping(value = "/Top/Logout", method = RequestMethod.GET)
    public String logout(HttpServletRequest request, Model model) {
    	
    	if (globalLoginSession.checkSessionState(request)) {
    		HttpSession session = request.getSession(true);
			session.setAttribute(LoginSession.LOGIN_STATE_KEY, LoginSession.LOGIN_STATE_DISABLE);
    	}
    	
    	globalLoginSession.setLoginAttr(model, LoginSession.ATTRIBUTE_SET_LOGOUT);
		
    	return "redirect:../Top";
    }
    
}
