package jp.ac.uaizu.rdbs.repository;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

import jp.ac.uaizu.rdbs.common.RDBSConstants.RequestSQL;
//import jp.ac.uaizu.rdbs.controller.DownloaderController;
import jp.ac.uaizu.rdbs.dao.MapFileInfoDao;

/**
 * Repository class of video playback function.
 */
@Repository
public class DownloadRepository {
	private Logger logger = LogManager.getLogger(DownloadRepository.class);
	
	@Autowired
	private JdbcTemplate jdbcTemplate;

	/**
	 * Get map information from uoa_4th_tbl using SQL
	 * @param robotId robot id
	 * @param location location
	 * @param dateFrom find date from
	 * @param dateTo find date to
	 * @return map file information
	 */
	public List<MapFileInfoDao> fetchMapInfo(String robotId, String location, String dateFrom, String dateTo){
		List<MapFileInfoDao> mapList = new ArrayList<MapFileInfoDao>();
		String setRobotId = "%" + robotId + "%";
		String setLocation = "%" + location + "%";
		Date setDateFrom = new Date(0);
		Date setDateTo = new Date();
		
//		setRobotId = "%" + robotId + "%";
//		setLocation = "%" + location + "%";
		if (dateFrom.length() != 0) {
			String timeDate = dateFrom + " 00:00:00";
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			try {
				setDateFrom = sdf.parse(timeDate);
			} catch (ParseException e) {
				setDateFrom = new Date(0);
				// TODO 自動生成された catch ブロック
				e.printStackTrace();
			}
			
		}
		if (dateTo.length() != 0) {
			String timeDate = dateTo + " 23:59:59";
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			try {
				setDateTo = sdf.parse(timeDate);
			} catch (ParseException e) {
				setDateTo = new Date();
				// TODO 自動生成された catch ブロック
				e.printStackTrace();
			}
			
		}
		
		logger.info(RequestSQL.MAP_SEARCH_CONDITION_QUERY);
		logger.info(setDateFrom.getTime()/1000);
		logger.info(setDateTo.getTime()/1000);
		
		List<Map<String, Object>> wk = jdbcTemplate.queryForList(RequestSQL.MAP_SEARCH_CONDITION_QUERY, 
				setRobotId, setLocation, (setDateFrom.getTime()/1000), (setDateTo.getTime()/1000));
		
		for (Map<String, Object> map : wk) {
			MapFileInfoDao item = new MapFileInfoDao();
			item.setRobotId((String)map.get("robot_id"));
			item.setLocation((String)map.get("location"));
			item.setDatetime((Date)map.get("datetime"));
			item.setYamlFilePath((String)map.get("map_yaml_path"));
			item.setPgmFilePath((String)map.get("map_pgm_path"));
			mapList.add(item);
		}
		return mapList;
	}
}

