/**
 * Start the download process
 */
function startDownload(){
	$(".btnDownload").prop("disabled", true);
	var json = {robot_id: $("#inRobotId").val(),
				location: $("#inLocation").val(),
				date_from: $("#inDateFrom").val(),
				date_to: $("#inDateTo").val()
	};

	if (param_validation(json)){
		/*set loading message*/
		dispLoading("Downloading...");
		console.log(json);
		$.ajax({
			type: "post",
			url: "download",
			data: json,
			dataType: "json"
	        }).then(function(mapZip){
				console.log("download list:" + Object.keys(mapZip).length);
				if (mapZip.count != 0){
					downloadFunc(mapZip);
					removeLoading();
					console.log("Download success end");
				}
				else{
					removeLoading();				
					alert("No matching map was found");
		        	console.log("Download failed");
				}
	        },
	        function(data, dataType) {
	        	removeLoading();
	        	alert("Download failed");
	        	console.log("Download failed");
	        });
	}
	$(".btnDownload").prop("disabled", false);
}

function param_validation(param){
	var result = false;
	
	if(validation_robotid(param)){
		if(validation_location(param)){
			if(validation_date(param)){
				result = true;
			}
		}
	}
	
	return result;
}

function validation_robotid(param){
	var result = false;
	var half_width_chars_ptn = new RegExp("^[0-9a-zA-Z\\_\\-]+$");
	var half_width_chars_match_res = param.robot_id.match(half_width_chars_ptn);
	if (param.robot_id.length <= 16){
		if (param.robot_id.length != 0){
			if (half_width_chars_match_res){
				result = true;
			}
			else{
				alert("Please enter single-byte alphanumeric characters");
			}
		}
		else{
			result = true;
		}
	}
	else{
		alert("Please enter up to 16 characters");
    }
	return result;
}

function validation_location(param){
	var result = false;
	var special_chars_ptn = new RegExp("^([^\"\'])*$");
	var special_chars_match_res = param.location.match(special_chars_ptn);
	if (special_chars_match_res){
		if (0 <= param.location.length && param.location.length <= 128){
			result = true;
		}
		else{
			alert("The maximum number of characters that can be entered is 128");
		}
    }
    else{
		alert("Contains invalid symbols");
    }
	return result;
}

function validation_date(param){
	var result = true;
	var date_diff = (new Date(param.date_to) - new Date(param.date_from)) / 86400000;
	if( Number.isNaN(date_diff) ){
		alert("Map search period is empty");
		result = false;
	}	
	else if ( date_diff < 0){
		alert("Map search period is not correct");
		result = false;
	}
	else if ( date_diff > 7){
		alert("Map search period is up to 7 days");
		result = false;
	}
	return result;
}

/**
 * Determine browser and sort download process
 * @param item
 */
function downloadFunc(item){
	var decodePath = decodeURIComponent(item.filePath);
	/*IE*/
    if (window.navigator.msSaveBlob){
    	downloadIE(item.fileName, decodePath);
    }
    /*chrome, firefox, safari*/
    else{
    	downloadOther(item.fileName, decodePath);
    }
}

/**
 * Wait for the specified millisecond
 * @param waitMsec Wait time (ms)
 */
function sleep(waitMsec){
	console.log("sleep:" + waitMsec);
	var startMsec = new Date();
	while(new Date() - startMsec < waitMsec);
}

/**
 * Download processing compatible with IE browsers
 * @param fileName
 * @param filePath
 */
function downloadIE(fileName, filePath){
	sleep(100);
	var xhr = new XMLHttpRequest();
	xhr.open('GET', filePath, true);
	xhr.responseType = 'blob';
	xhr.onreadystatechange = function(){
	    if (xhr.readyState === XMLHttpRequest.DONE && xhr.status === 200) {
	    	window.navigator.msSaveBlob(xhr.response, fileName);
	    }
	}
	xhr.send();
}

/**
 * Download processing compatible with chrome, firefox, and safari browsers
 * @param fileName
 * @param filePath
 */
function downloadOther(fileName, filePath){
	sleep(100);
	var a = document.createElement('a');
	var div = document.createElement('div');
	div.className = 'downloadClass';
    a.download = fileName;
    a.href = filePath;
    a.style.cursor = 'pointer';
    a.type ='application/octet-stream';
    
    div.appendChild(a);
    document.body.appendChild(div);
    a.click();
    div.removeChild(a);
    document.body.removeChild(div);
}

/**
 * Output downloading message
 * @param msg
 */
function dispLoading(msg){
	if( msg == undefined ){
		msg = "";
	}
//	var spinner_elem = '<div class="progress-bar progress-bar-success progress-bar-striped active" role="progressbar" style="margin-left:25; margin-right:25; width: 70%; height:20%;">' + msg + '</div>';
//	var dispMsg = '<div class="loadingMsg offset-md-3 col-md-8" style="margin-left:25; margin-right:25;">' + spinner_elem + '</div>';
//	$('body').append('<div id="loading" style="width:100%; height:100%; position:fixed; top:0; left:0; background-color:#fff; opacity:0.8;">' + dispMsg + '</div>');
	var spinner_elem = '<div class="col-md-7 progress-bar progress-bar-striped active" role="progressbar" style="margin-left:25; margin-right:25; width: 80%; height:20%;">' + msg + '</div>';
	var dispTopSide = '<div style="width:100%; height:50%; top:0; left:0; position:fixed; background-color:#fff; opacity:0.8;"></div>';
	var dispUnderSide = '<div style="width:100%; height:50%; position:fixed; background-color:#fff; opacity:0.8;"></div>';
	var dispMsg = '<div class="loadingMsg row" style="background-color:#fff; opacity:0.8;"><div class="col-md-1"></div>' + spinner_elem + '<div class="col-md-1"></div></div>';
	$('body').append('<div id="loading">' + dispTopSide + dispMsg + dispUnderSide + '</div>');
}

/**
 * Remove downloading message
 * @returns
 */
function removeLoading(){
	$("#loading").remove();
}